/*---------------------------------------------------------------
 *	物理タイマを使ったドレミファ再生(μT-Kernel 3.0用)
 *
 *	Copyright (C) 2022-2023 by T3 WG of TRON Forum
 *---------------------------------------------------------------*/
#include <tk/tkernel.h>
#include <tm/tmonitor.h>

//-------- GPIO関連の共通処理 -----------------------------------
// GPIOの特定のピンpinの出力をval(0または1)に設定
//	pinで指定された以外のピンの状態は変化しない
LOCAL void out_gpio_pin(UW port, UW pin, UW val)
{
	INT port_addr;

	if(port == 1){							// P1の場合
		if(val)
			port_addr = GPIO(P1, OUTSET);	// P1でvalが1の場合
		else							
			port_addr = GPIO(P1, OUTCLR);	// P1でvalが0の場合

	} else {								// P0の場合
		if(val)
			port_addr = GPIO(P0, OUTSET);	// P0でvalが1の場合
		else
			port_addr = GPIO(P0, OUTCLR);	// P0でvalが0の場合
	}
	out_w(port_addr, (1 << pin));			// 指定のピンに1を出力
}

//-------- スピーカーへの出力 -----------------------------------
LOCAL BOOL	speaker_val = FALSE;		// 現在のスピーカーへの出力値

// スピーカーへの出力値の0と1を反転する関数(※F)
void speaker_onoff(void)
{
	// スピーカーに接続されたGPIOピン(P0.00)にspeaker_valの値を出力
	out_gpio_pin(0, 0, speaker_val);	// port=0, pin=0を指定(※G)

	// 次の出力のために値を反転(※J)
	speaker_val = (! speaker_val);
}

//-------- 物理タイマを使ったドレミファ再生 ---------------------

// 物理タイマ関連情報
const T_DPTMR dptmr = {0, TA_HLNG, &speaker_onoff};
const UINT ptmrno = 1;				// 物理タイマ番号として1を使用
const INT ptmr_clk_mhz = 16;		// 物理タイマのクロック(MHz単位)

// 周波数freq(Hz)の音をplay_time(ミリ秒)の間だけスピーカーから再生(※S)
LOCAL void play_speaker(INT freq, INT play_time)
{
	// 物理タイマハンドラの起動周期時間(マイクロ秒単位)と上限値の計算(※T)
	INT cycle_us = 1000000 / freq / 2;				// 起動周期時間
	INT limit = cycle_us * ptmr_clk_mhz - 1;		// 物理タイマの上限値

	StartPhysicalTimer(ptmrno, limit, TA_CYC_PTMR);	// 物理タイマの動作開始(※U)
    tm_printf("play_speaker_ptimer: freq=%4d, play_time=%4d\n",
				  freq, play_time);
	tk_dly_tsk(play_time);							// 再生時間だけ待つ(※V)
	StopPhysicalTimer(ptmrno);
}

// ドレミファソラシド(C5,D5,E5,F5,G5,A5,B5,C6)の周波数(Hz)の定義(※W)
const INT doremi[] = { 523, 587, 659, 698, 783, 880, 987, 1046 };

EXPORT void usermain(void)
{
	// スピーカーに接続されたGPIOピン(P0.00)を出力に設定
	out_w(GPIO(P0, PIN_CNF(0)), (1 << 0));
	
	DefinePhysicalTimerHandler(ptmrno, &dptmr);		// 物理タイマハンドラ定義
	for (INT cnt = 0; cnt < 8; cnt++)
		play_speaker(doremi[cnt], 500);		// 各音階を500ミリ秒ずつ再生(※X)
	tk_slp_tsk(TMO_FEVR);					// 永久待ち
}
